#= require spec_helper
#= require slender_data/cache


describe 'Cache', ->

  beforeEach ->
    @cache = new slender_data.Cache()


  describe 'add', ->

    it 'stores the given data in the cache', ->
      @cache.add 3, 'entry'
      expect(@cache.get 3).to.eql 'entry'

    it 'overwrites existing entries', ->
      @cache.add 3, 'entry'
      @cache.add 3, 'new entry'
      expect(@cache.get 3).to.equal 'new entry'


  describe 'get', ->

    it "returns the entry if it exists", ->
      @cache.add 3, 'entry'
      expect(@cache.get 3).to.equal 'entry'

    it 'returns undefined if the entry does not exist', ->
      expect(@cache.get 4).to.be.undefined


  describe 'get_many', ->

    beforeEach ->
      @cache.add 1, 'one'
      @cache.add 2, 'two'
      @result = @cache.get_many [1, 3]

    it "returns the values that exist in the 'found' array", ->
      expect(@result.found).to.eql ['one']

    it "returns the keys that don't exist in the 'missing' array", ->
      expect(@result.missing).to.eql [3]


  describe 'length', ->

    it 'returns 0 for empty cache', ->
      expect(@cache.length()).to.equal 0

    it 'returns the number of objects in the cache', ->
      @cache.add 1, 'one'
      expect(@cache.length()).to.equal 1
      @cache.add 2, 'two'
      expect(@cache.length()).to.equal 2


  describe 'remove', ->

    it 'removes the entry with the given key from the chache', ->
      @cache.add 2, 'two'
      @cache.remove 2
      expect(@cache.length()).to.equal 0

    it 'only removes the given data and leaves the rest of the cache alone', ->
      @cache.add 1, 'one'
      @cache.add 2, 'two'
      @cache.remove 1
      expect(@cache.length()).to.equal 1
      expect(@cache.get 1).to.be.undefined
      expect(@cache.get 2).to.equal 'two'


    it 'can handle non-existing keys', ->
      @cache.remove 99
      expect(@cache.length()).to.equal 0


  describe 'remove_many', ->

    it 'removes all entries with matching keys from the chache', ->
      @cache.add 1, 'one'
      @cache.add 2, 'two'
      @cache.add 3, 'three'
      @cache.remove_many [1, 3]
      expect(@cache.length()).to.equal 1
      expect(@cache.get 2).to.equal 'two'

    it 'can handle non-existing keys', ->
      @cache.remove_many [1, 3]
      expect(@cache.length()).to.equal 0

