#= require spec_helper
#= require slender_data/loading_tracker


describe 'slender_data.LoadingTracker', ->

  beforeEach ->
    define_test_data this
    @loading_tracker = new slender_data.LoadingTracker @test_params


  describe 'entry_arrived', ->

    beforeEach ->
      @loading_tracker.start_loadings users: [@user_1, @user_2]

    it 'calls the queued callback with the loaded entry', (done) ->
      @loading_tracker.wait_for_entry_loaded user: @user_1, (entry) =>
        expect(entry).to.equal @user_1
        done()
      @loading_tracker.entry_arrived @user_1

    it 'unregisters the arrived entrys from the loading list', ->
      @loading_tracker.entry_arrived @user_1
      expect(@loading_tracker).to.not.have.loading user: @user_1

    it 'does not unregister other loading entries', ->
      @loading_tracker.entry_arrived @user_1
      expect(@loading_tracker).to.have.loading user: @user_2


  describe 'is_loading', ->

    it 'returns false if the entry is not loading', ->
      expect(@loading_tracker.is_loading id: 3).to.be.false

    it 'returns true if the entry is currently loading', ->
      @loading_tracker.start_loading id: 3
      expect(@loading_tracker.is_loading id: 3).to.be.true


  describe 'start_loading', ->

    it 'stores the id of the loading query', ->
      @loading_tracker.start_loading @user_1
      expect(@loading_tracker).to.have.loading user: @user_1


  describe 'start_loadings', ->

    it 'stores the ids of the loading entries', ->
      @loading_tracker.start_loadings users: [@user_1, @user_3]
      expect(@loading_tracker).to.have.loading user: @user_1
      expect(@loading_tracker).to.not.have.loading user: @user_2
      expect(@loading_tracker).to.have.loading user: @user_3


  describe 'wait_for_entry_loaded', ->

    describe 'without a request running', ->

      beforeEach ->
        @callback = sinon.stub()
        @result = @loading_tracker.wait_for_entry_loaded user: @user_1,
                                                         @callback

      it 'returns the given query as the function result', ->
        expect(@result).to.eql_query user: @user_1

      it 'does not call the given callback', ->
        expect(@callback).to.not.have.been.called

      it 'does not call the given callback when the entry gets loaded later', ->
        @loading_tracker.entry_arrived @user_1
        expect(@callback).to.not.have.been.called


    describe 'while a request is running', ->

      beforeEach ->
        @loading_tracker.start_loading user: @user_1
        @callback = sinon.stub()
        @result = @loading_tracker.wait_for_entry_loaded user: @user_1,
                                                         @callback

      it 'returns nothing', ->
        expect(@result).to.be.undefined

      it 'does not call the given callback', ->
        expect(@callback).to.not.have.been.called

      it 'calls the given callback when the entry is loaded later', ->
        @loading_tracker.entry_arrived @user_1
        expect(@callback).to.have.been.calledOnce


  describe 'wait_for_entries_loaded', ->

    beforeEach ->
      @loading_tracker.start_loadings users: [@user_1, @user_2]
      @callback = sinon.stub()
      @result = @loading_tracker.wait_for_entries_loaded users: [@user_1, @user_2, @user_3],
                                                         @callback

    it 'returns a multiquery describing the entries that are not currently loading', ->
      expect(@result).to.eql_query users: [@user_3]

    it 'does not call the callback if not all entries have been loaded yet', ->
      expect(@callback).to.not.have.been.called
      @loading_tracker.entry_arrived @user_1
      expect(@callback).to.not.have.been.called

    it 'calls the given callback with the loaded entries when they all have loaded', ->
      @loading_tracker.entry_arrived @user_1
      @loading_tracker.entry_arrived @user_2
      expect(@callback).to.have.been.calledOnceWith [@user_1, @user_2]

