#= require spec_helper


describe 'Parameterizable', ->

  beforeEach ->
    define_test_data this


  describe 'constructor', ->

    it 'stores the given parameters for later use', ->
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.params).to.not.be.undefined

    it 'allows to configure the name of data models', ->
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.entry_name).to.equal 'user'

    it 'defaults the name for a collection of data models to the plural version of the name of a single data model', ->
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.entries_name).to.equal 'users'

    it 'allows to configure the name for a collection of data models', ->
      @test_params.entries_name = 'humans'
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.entries_name).to.equal 'humans'

    it 'uses the "id" attribute as an identifier by default', ->
      @test_params.id_name = undefined
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.id_name).to.equal 'id'

    it 'allows to configure which attribute is used as the indentifier', ->
      @test_params.id_name = 'foo'
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.id_name).to.equal 'foo'

    it 'defaults the name of a collection of identifiers to "ids"', ->
      @test_params.id_name = undefined
      @test_params.ids_name = undefined
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.ids_name).to.equal 'ids'

    it 'allows to configure the name of a collection of identifiers', ->
      @test_params.ids_name = 'foo'
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.ids_name).to.equal 'foo'

    it 'uses the "version" attribute for versioning by default', ->
      @test_params.version_name = undefined
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.version_name).to.equal 'version'

    it 'allows to configure which attribute is used for versioning', ->
      @test_params.version_name = 'foo'
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.version_name).to.equal 'foo'

    it 'does not perform versioning by default', ->
      @test_params.versioning = undefined
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.versioning).to.be.false

    it 'allows to enable versioning', ->
      @test_params.versioning = yes
      @parameterizable = new slender_data.Parameterizable @test_params
      expect(@parameterizable.versioning).to.be.true



  describe 'querify_single', ->

    beforeEach ->
      @test_params.versioning = yes
      @test_params.partial_entries = yes
      @parameterizable = new slender_data.Parameterizable @test_params
      @query_hash = id: 3, full: yes

    context 'given a query hash', ->

      beforeEach ->
        @result = @parameterizable._querify_single @query_hash

      it 'returns a query instance for the given hash', ->
        expect(@result).to.be.an.instanceof slender_data.SingleQuery
        expect(@result).to.eql_query @query_hash

      it 'configures the created query with the default configuration values of the current query', ->
        expect(@result.versioning).to.be.true
        expect(@result.partial_entries).to.be.true

      it 'configures the created query with teh query parameters of the current query', ->
        expect(@result.full).to.be.true

    context 'given a query object', ->

      beforeEach ->
        @query = new slender_data.SingleQuery @query_hash, @test_params
        @result = @parameterizable._querify_single @query

      it 'returns a given query directly', ->
        expect(@result).to.equal @query


  describe 'querify_multi', ->

    beforeEach ->
      @test_params.versioning = yes
      @test_params.partial_entries = yes
      @parameterizable = new slender_data.Parameterizable @test_params
      @query_hash = ids: [3, 4], full: yes

    context 'given a query hash', ->

      beforeEach ->
        @result = @parameterizable._querify_multi @query_hash

      it 'returns a query instance for the given hash', ->
        expect(@result).to.be.an.instanceof slender_data.MultiQuery
        expect(@result).to.eql_query @query_hash

      it 'configures the created query with the default configuration values of the current query', ->
        expect(@result.versioning).to.be.true
        expect(@result.partial_entries).to.be.true

      it 'configures the created query with teh query parameters of the current query', ->
        expect(@result.full).to.be.true

    context 'given a query object', ->

      beforeEach ->
        @query = new slender_data.MultiQuery @query_hash, @test_params
        @result = @parameterizable._querify_multi @query

      it 'returns a given query directly', ->
        expect(@result).to.equal @query

