#= require spec_helper
#= require slender_data/queries/single_query


describe 'slender_data.SingleQuery', ->

  beforeEach ->
    define_test_data this
    @entry_query = new slender_data.SingleQuery user: @user_1, @test_params
    @id_query = new slender_data.SingleQuery username: 'one', @test_params


  describe 'constructor', ->

    it 'allows to provide an id using its generic name "id"', ->
      query = new slender_data.SingleQuery id: 'one', @test_params
      expect(query.id()).to.eql 'one'

    it 'allows to provide an id using its domain specific name (i.e. "username")', ->
      query = new slender_data.SingleQuery username: 'one', @test_params
      expect(query.id()).to.eql 'one'

    it 'allows to provide an entry using its generic name "entry"', ->
      query = new slender_data.SingleQuery entry: {username: 'one'}, @test_params
      expect(query.entry()).to.eql username: 'one'

    it 'allows to provide an entry using its domain specific name (i.e. "user")', ->
      query = new slender_data.SingleQuery user: {username: 'one'}, @test_params
      expect(query.entry()).to.eql username: 'one'

    it 'allows to provide the version using its generic name "version"', ->
      query = new slender_data.SingleQuery entry: {username: 'foo', version: 2}, @test_params
      expect(query.version()).to.eql 2

    it 'allows to provide an entry using its domain specific name (i.e. "timestamp")', ->
      query = new slender_data.SingleQuery user: {username: 'foo', timestamp: 2}, @test_params
      expect(query.version()).to.eql 2

    it 'throws an exception for missing query selectors', ->
      expect(=> new slender_data.SingleQuery 'one', @test_params).to.throw()

    it 'throws an exception for unknown query selectors', ->
      expect(=> new slender_data.SingleQuery zonk: 'one', @test_params).to.throw()



  describe 'entry getter', ->

    context 'entry query', ->

      it 'allows access through the entry() function', ->
        expect(@entry_query.entry()).to.eql @user_1

      it 'allows access through the domain-specific name', ->
        expect(@entry_query.user()).to.eql @user_1


    context 'id query', ->

      it 'throws an exception', ->
        expect(-> @id_query.user()).to.throw()



  describe 'id getter', ->

    context 'entry query', ->

      it 'returns the id through the id() function', ->
        expect(@entry_query.id()).to.equal 'one'

      it 'returns the id through its domain-specific name', ->
        expect(@entry_query.username()).to.equal 'one'


    context 'id query', ->

      it 'returns the id through the id() function', ->
        expect(@id_query.id()).to.equal 'one'

      it 'returns the id through its domain-specific name', ->
        expect(@id_query.username()).to.equal 'one'



  describe 'version getter', ->

    context 'entry query', ->

      it 'returns the version through the version() function', ->
        expect(@entry_query.version()).to.equal 2

      it 'returns the version through its domain-specific name', ->
        expect(@entry_query.timestamp()).to.equal 2


    context 'id query', ->

      it 'throws an exception because id queries can not contain versions', ->
        expect(-> @id_query.version).to.throw()
        expect(-> @id_query.timestamp).to.throw()

