#= require jquery
#= require sinon-chai
#= require sinon
#= require slender_data
#= require underscore
#= require asynchronizer

@define_test_data = (that) ->
  that.user_1 = username: 'one', email: 'one@example.com', timestamp: 2
  that.user_2 = username: 'two', email: 'two@example.com', timestamp: 2
  that.user_3 = username: 'three', email: 'three@example.com', timestamp: 2
  that.partial_user_1 = username: 'one', version: 2
  that.full_user_1 = username: 'one', version: 2, email: 'one@example.com'
  that.user_1_outdated = username: 'one', timestamp: 1
  that.partial_user_2 = username: 'two', version: 2
  that.full_user_2 = username: 'two', version: 2, email: 'two@example.com'
  that.partial_user_3 = username: 'three', version: 2
  that.full_user_3 = username: 'three', version: 2, email: 'three@example.com'

  that.test_params =
    entry_name: 'user'
    id_name: 'username'
    version_name: 'timestamp'
    full_entry_checker: (entry) -> entry?.email


# Domain-specifc matcher for checking if the tested callback stub
# has been called once with the given arguments.
chai.Assertion.addMethod 'calledOnceWith', (argument) ->
  new chai.Assertion(@_obj).to.have.been.calledOnce
  new chai.Assertion(@_obj).to.have.been.calledWith argument


# Domain-specifc matcher for checking if the tested callback
# has been called once and with the given Query object.
chai.Assertion.addMethod 'calledOnceWithQuery', (query) ->
  expect(@_obj).to.have.been.calledOnce
  expect(@_obj.args[0][0].to_hash()).to.eql query


# Domain-specifc matcher for checking if the tested entry
# is a readble copy of the given server entry.
chai.Assertion.addMethod 'editable_version_of', (server_entry) ->
  new chai.Assertion(@_obj, 'Should be an editable version').to.eql server_entry
  new chai.Assertion(@_obj, 'Should be an editable version').to.not.equal server_entry


# Domain-specifc matcher for checking if the tested entry
# is a readble copy of the given server entry.
chai.Assertion.addMethod 'eql_query', (query) ->
  new chai.Assertion(@_obj.to_hash(), 'Should be query equal to').to.eql query


# Domain-specifc matcher for checking if the given entry is currently loading.
chai.Assertion.addMethod 'loading', (query) ->
  @assert @_obj.is_loading(query),
          """expected entry '#{query.username}' to be loading,
             but it was not""",
          """expected entry '#{query.username}' to not be loading,
             but it is"""
